@@ -1,8 +1,11 @@
 use clap::{app_from_crate, App, Arg};
+#[cfg(feature = "completions")]
+use clap_complete::{generate, Generator};
+#[cfg(feature = "completions")]
+use std::io;
 
 pub fn build_cli<'a>(possible_styles: &'a [&'a str]) -> App<'a> {
-    app_from_crate!()
-        // .settings(&[AppSettings::ArgRequiredElseHelp, AppSettings::ValidArgFound])
+    let app = app_from_crate!()
         .about(
             "Control your scripts over the network.
 
@@ -68,11 +71,16 @@ $ echo 'Option 1\\nOption 2' | thqm -U |
                 .takes_value(true)
                 .possible_values(possible_styles),
         )
+        .arg(
+            Arg::new("list_styles")
+                .help("List available page styles.")
+                .long("list-styles"),
+        )
         .arg(
             Arg::new("interface")
                 .help("Network interface to use to determine ip.")
                 .long("interface")
-                .takes_value(true)
+                .takes_value(true),
         )
         .arg(
             Arg::new("show_qrcode")
@@ -116,5 +124,23 @@ $ echo 'Option 1\\nOption 2' | thqm -U |
                 .help("Don't show the qrcode on the page.")
                 .long("no-qrcode")
                 .takes_value(false),
-        )
+        );
+
+    #[cfg(feature = "completions")]
+    let app = app.arg(
+        Arg::new("completions")
+            .help("Generate shell completions.")
+            .long("completions")
+            .value_name("shell")
+            .takes_value(true)
+            .possible_values(["bash", "elvish", "fish", "powershell", "zsh"]),
+    );
+
+    app
+}
+
+#[cfg(feature = "completions")]
+pub fn print_completions<G: Generator>(gen: G, app: &mut App) {
+    // TOOD: make completion look for installed styles.
+    generate(gen, app, app.get_name().to_string(), &mut io::stdout());
 }
